<?php

/**
 * @package CsvImporterPlusforACF
 */

namespace CIPFA\base;

use League\Csv\Reader;


// make sure we don't expose any info if called directly
if (!defined('ABSPATH')) exit; // Exit if accessed directly

class CIPFA_CsvImporter
{

   public static function cipfa_register()
   {
      add_action('init', array(__CLASS__, 'cipfa_initialize_import_count'));
      add_action('wp_ajax_cipfa_data', array(__CLASS__, 'cipfa_data_ajax'));
   }

   // Initialize total imported posts count option
   public static function cipfa_initialize_import_count()
   {
      if (get_option('cipfa_total_imported_so_far') === false) {
         update_option('cipfa_total_imported_so_far', 0);
      }
   }
   // Function to handle CSV import AJAX request
   public static function cipfa_data_ajax()
   {
      // Check nonce
      check_ajax_referer('cipfa_nonce', 'cipfa_nonce_field');

      $mapping = isset($_POST['mapping']) ? array_map('sanitize_text_field', wp_unslash($_POST['mapping'])) : array();

      // Get selected post type
      $post_type = isset($_POST['post_type']) ? sanitize_text_field(wp_unslash($_POST['post_type'])) : 'post';

      // Get current chunk index
      $current_chunk = isset($_POST['current_chunk']) ? intval($_POST['current_chunk']) : 0;

      // Get the edit_existing flag
      $edit_existing = isset($_POST['edit_existing']) && $_POST['edit_existing'] === '1';

      // Get the flush_cache flag
      $flush_cache = isset($_POST['flush_cache']) && $_POST['flush_cache'] === '1';

      // Get chunk size from request or default to 10
      $chunk_size = isset($_POST['chunk_size']) ? intval($_POST['chunk_size']) : 10;
      $total_imported = 0;

      // Get CSV File
      $csv_file = get_option('cipfa_csv_file_path');

      // Get CSV data and mapping
      if (!isset($csv_file) || !file_exists($csv_file)) {
         wp_die(esc_html__('CSV file is missing.', 'csv-importer-plus-for-acf'));
      }

      // Reset total imported posts count at the start of a new import process
      if ($current_chunk === 0) {
         CIPFA_Logs::cipfa_clear_logs();
         update_option('cipfa_total_imported_so_far', 0);
         self::cipfa_find_and_store_duplicates($csv_file);
      }

      // Get total imported posts count from options
      $cipfa_total_imported_so_far = get_option('cipfa_total_imported_so_far', 0);

      // Get total records and total chunks
      $csv = Reader::createFromPath($csv_file, 'r');
      $csv->setHeaderOffset(0);
      $total_records = iterator_count($csv->getRecords());
      $total_chunks = ceil($total_records / $chunk_size);

      // Reset the CSV pointer to the beginning after counting records
      $csv = Reader::createFromPath($csv_file, 'r');
      $csv->setHeaderOffset(0);

      // Get the current chunk
      $iterator = $csv->getRecords();
      $limitIterator = new \LimitIterator($iterator, $current_chunk * $chunk_size, $chunk_size);

      // Process the current chunk
      $chunk = [];
      foreach ($limitIterator as $record) {
         $chunk[] = $record;
      }

      // Process the chunk if it's not empty
      if (!empty($chunk)) {

         // Process the chunk
         $total_imported_for_chunk = CIPFA_PostProcessor::cipfa_process_chunk($chunk, $mapping, $post_type, $edit_existing);
         $total_imported += $total_imported_for_chunk;

         // Update total imported count
         $cipfa_total_imported_so_far += $total_imported_for_chunk;

         // Save the updated total imported count back to options
         update_option('cipfa_total_imported_so_far', $cipfa_total_imported_so_far);

         // Increment the current chunk counter
         $current_chunk++;
      }

      $logs = CIPFA_Logs::cipfa_get_logs();
      $html_log_list = '';
      if (!empty($logs)) {
         // Format the error logs as an HTML list
         $html_log_list = '<ul class="list-disc pl-5">';
         foreach ($logs as $log) {
            $html_log_list .= '<li class="text-black text-sm">' . esc_html($log) . '</li>';
         }
         $html_log_list .= '</ul>';
      }

      // Check if this is the last chunk
      $is_last_chunk = ($current_chunk) >= $total_chunks;

      // If it is the last chunk, reset the total imported count
      if ($is_last_chunk) {
         CIPFA_Logs::cipfa_clear_logs();
         update_option('cipfa_total_imported_so_far', 0);
         update_option('cipfa_csv_file_path', '');
         delete_option('cipfa_duplicate_media');
         delete_option('cipfa_protocol_mismatch_log_shown');
         delete_option('cipfa_external_url_log_shown');
         if (function_exists('wc_update_product_lookup_tables') && $post_type === 'product') {
            wc_update_product_lookup_tables();
         }
         if ($flush_cache) {
            wp_cache_flush();
         }
      }

      $success_message_template = esc_html__('posts imported successfully! Total chunks imported: ', 'csv-importer-plus-for-acf');
      $success_message = intval($cipfa_total_imported_so_far) . ' ' . $success_message_template . intval($total_chunks);

      // Send response back to the client after the chunk is processed
      wp_send_json_success(array(
         'cipfa_total_imported_so_far' => intval($cipfa_total_imported_so_far),
         'total_chunks' => intval($total_chunks),
         'current_chunk' => intval($current_chunk),
         'chunk_size' => intval($chunk_size),
         'message' => sanitize_text_field($success_message),
         'logs' => wp_kses_post($html_log_list)
      ));
   }

   //store duplicate media
   public static function cipfa_find_and_store_duplicates($csv_file)
   {
      // Create a CSV reader instance from the file path
      $csv = Reader::createFromPath($csv_file, 'r');

      // Define the file pattern to match media URLs
      $file_pattern = '/\.(jpg|jpeg|png|gif|bmp|webp|ico|svg|txt|pdf|doc|docx|ppt|pptx|pps|ppsx|odt|xls|xlsx|mp3|m4a|ogg|wav|mp4|m4v|mov|wmv|avi|mpg|ogv|3gp|3g2|webm|zip|tar|gz|ttf|otf|woff|woff2|csv|json)$/i';

      // Arrays to store media URLs and duplicates
      $media_urls = [];
      $duplicate_media = [];

      // Iterate through the CSV rows
      foreach ($csv as $row) {
         foreach ($row as $value) {
            $value = trim($value);

            // Check for media duplicates
            if (preg_match($file_pattern, $value)) {
               if (isset($media_urls[$value])) {
                  // If the URL is already in the array, it's a duplicate
                  $duplicate_media[$value] = '';
               } else {
                  // Otherwise, add it to the array
                  $media_urls[$value] = true;
               }
            }
         }
      }

      // Store the duplicate media in the options table (WordPress specific)
      if (!empty($duplicate_media)) {
         update_option('cipfa_duplicate_media', $duplicate_media, false);
      }

      // Return the duplicate media array
      return [
         'media' => $duplicate_media,
      ];
   }
}
