<?php

/**
 * @package CsvImporterPlusforACF
 */

namespace CIPFA\base;

// make sure we don't expose any info if called directly
if (!defined('ABSPATH')) exit; // Exit if accessed directly

use DateTime;

class CIPFA_PostProcessor
{

   // Function to process a chunk of CSV data and return the number of posts imported
   public static function cipfa_process_chunk($chunk, $mapping, $post_type, $edit_existing)
   {
      // Initialize total imported count for this chunk
      $total_imported_chunk = 0;

      // Loop through chunk
      foreach ($chunk as $row) {
         // Process row and update total imported count for this chunk
         $total_imported_chunk += self::cipfa_process_row($row, $mapping, $post_type, $edit_existing);
      }

      return $total_imported_chunk; // Return total imported count for this chunk
   }

   // Function to process a row of CSV data and return the number of posts imported
   public static function cipfa_process_row($row, $mapping, $post_type, $edit_existing)
   {
      // Step 1: Prepare the post data
      $post_data = self::cipfa_prepare_post_data($mapping, $row, $post_type);

      // Keys to check and map to variables
      $keys_to_check = [
         'acf_fields',
         'post_category',
         'post_content',
         'post_excerpt',
         'taxonomy_terms',
         'post_tags',
         'post_featured_image',
         'csv_post_id',
         'yoast_fields',
         'product_fields',
         'product_image_gallery'
      ];

      // Dynamically set variables if the keys exist in $post_data
      foreach ($keys_to_check as $key) {
         if (array_key_exists($key, $post_data)) {
            $$key = $post_data[$key];
         }
      }

      // Step 2: Handle the post author
      $post_data['post_author'] = self::cipfa_handle_post_author(
         $post_data['post_author'],
         isset($post_data['post_author_email']) ? $post_data['post_author_email'] : null
      );

      // SKU already exists
      if ($post_type === 'product' && !empty($product_fields['_sku'])) {
         $sku = trim($product_fields['_sku']);

         $products = wc_get_products([
            'sku'    => $sku,
            'limit'  => -1,
            'return' => 'ids',
         ]);

         // Ensure exact SKU match
         $exact_product = array_filter($products, function ($product_id) use ($sku) {
            $product = wc_get_product($product_id);
            return $product && strtolower($product->get_sku()) === strtolower($sku);
         });

         if (!empty($exact_product)) {
            CIPFA_Logs::cipfa_Log('A product with this exact SKU already exists. SKU: ' . $sku);
            return;
         }
      }

      // Step 3: Insert or update the post
      $post_id = self::cipfa_insert_or_update_post($post_data, $edit_existing, $csv_post_id ?? null);

      // If post insertion or update failed, return 0
      if ($post_id == 0) {
         return 0;
      }

      // Check steps dynamically
      if (isset($post_featured_image)) {
         // Step 4: Set featured image
         self::cipfa_set_featured_image($post_id, $post_featured_image);
      }

      if (isset($post_category)) {
         // Step 5: Set categories
         self::cipfa_set_categories($post_id, $post_category);
      }

      if (isset($post_content)) {
         // Step 6: Handle images in post content
         self::cipfa_handle_images_in_post_field($post_id, 'post_content', $post_content);
      }

      if (isset($post_excerpt)) {
         // Step 7: Handle images in post excerpt
         self::cipfa_handle_images_in_post_field($post_id, 'post_excerpt', $post_excerpt);
      }

      if (isset($taxonomy_terms)) {
         // Step 8: Set custom taxonomies
         self::cipfa_set_custom_taxonomy_terms($post_id, $taxonomy_terms);
      }

      if (isset($post_tags)) {
         // Step 9: Set tags
         self::cipfa_set_tags($post_id, $post_tags);
      }

      if (isset($acf_fields)) {
         // Step 10: Update ACF fields
         self::cipfa_update_acf_fields($post_id, $acf_fields);
      }

      if (isset($yoast_fields)) {
         // Step 11: Update Yoast fields
         self::cipfa_update_yoast_fields($post_id, $yoast_fields);
      }

      if (isset($product_fields)) {

         // Determine active price
         self::cipfa_update_product_price($product_fields);

         // Set Product Image Gallery
         if (!empty($product_image_gallery)) {
            $product_fields['_product_image_gallery'] = self::cipfa_set_product_image_gallery($post_id, $product_image_gallery);
         }

         // Check if file URLs are provided
         if (!empty($product_fields['download_file_url'])) {
            self::cipfa_set_product_downloadable_files($product_fields);
         }

         // Set Attributes
         self::cipfa_set_product_attributes($post_id, $product_fields);

         // Step 12: Update Product fields
         self::cipfa_update_product_fields($post_id, $product_fields);
      }

      return 1; // Return 1 as one row has been processed
   }

   //Prepare Post Data
   private static function cipfa_prepare_post_data($mapping, $row, $post_type)
   {
      $post_data = array(
         'post_type' => $post_type,
         'post_author' => '',
      );

      $product_fields = array();
      $product_image_gallery = '';
      $acf_fields = array();
      $yoast_fields = array();
      $post_tags = array();
      $post_category = array();
      $post_content = array();
      $post_excerpt = array();
      $post_featured_image = '';
      $csv_post_id = '';
      $taxonomy_terms = array();

      // Retrieve post data from CSV based on the mapping
      foreach ($mapping as $column_name => $field_key) {
         // Retrieve the value from the row
         $column_value = $row[$column_name] ?? '';
         // Retrieve attribute name
         $attribute = CIPFA_PostProcessor::cipfa_extract_attribute($column_name);

         switch ($field_key) {
            case 'product_name':
            case 'post_title':
               $post_data['post_title'] = sanitize_text_field($column_value);
               break;
            case 'product_date':
            case 'post_date':
               $post_data['post_date'] = self::cipfa_set_date_based_on_formats($column_value);
               break;
            case 'product_description':
            case 'post_content':
               $post_content = wp_kses_post($column_value);
               break;
            case 'product_short_description':
            case 'post_excerpt':
               $post_excerpt = wp_kses_post($column_value);
               break;
            case 'product_image':
            case 'featured_image':
               $post_featured_image = esc_url_raw($column_value);
               break;
            case 'post_category':
               $post_category = array_map('sanitize_text_field', explode(',', $column_value));
               break;
            case 'tags':
               $post_tags = array_map('sanitize_text_field', explode(',', $column_value));
               break;
            case 'post_author':
               $post_data['post_author'] = sanitize_text_field($column_value);
               break;
            case 'post_author_email':
               $post_data['post_author_email'] = sanitize_email($column_value);
               break;
            case 'author_password':
               $post_data['author_password'] = sanitize_text_field($column_value);
               break;
            case 'post_parent':
               if (self::cipfa_is_valid_post_parent($column_value)) {
                  $post_data['post_parent'] = intval($column_value);
               }
               break;
            case 'product_position':
            case 'menu_order':
               $post_data['menu_order'] = intval($column_value);
               break;
            case 'page_template':
               $template_key = self::cipfa_is_valid_page_template($column_value);
               if ($template_key) {
                  $post_data['page_template'] = sanitize_text_field($template_key);
               }
               break;
            case 'product_published':
            case 'post_status':
               $post_data['post_status'] = sanitize_text_field($column_value);
               break;
            case 'post_password':
               $post_data['post_password'] = sanitize_text_field($column_value);
               break;
            case 'comments':
               $post_data['comment_status'] = strtolower(sanitize_text_field($column_value)) === 'disable' ? 'closed' : 'open';
               break;
            case 'pings':
               $post_data['ping_status'] = strtolower(sanitize_text_field($column_value)) === 'disable' ? 'closed' : 'open';
               break;
            case 'product_id':
            case 'post_id':
               $csv_post_id = intval($column_value);
               break;
            case 'product_brand':
            case 'product_visibility':
            case 'product_type':
            case 'product_cat':
            case 'product_tag':
            case 'product_shipping_class':
            case taxonomy_exists($column_name):
               if ($field_key === 'product_visibility') {
                  $allowed_visibility = [
                     'visible'                => 'visible',
                     'featured'               => 'featured',
                     'feature'                => 'featured',
                     'exclude from catalog'   => 'exclude-from-catalog',
                     'exclude from search'    => 'exclude-from-search',
                     'exclude-from-catalog'   => 'exclude-from-catalog',
                     'exclude-from-search'    => 'exclude-from-search',
                  ];
                  $values = [trim($column_value)];
                  $mapped_terms = array_map(function ($value) use ($allowed_visibility) {
                     $clean = strtolower(sanitize_text_field($value));
                     return $allowed_visibility[$clean] ?? 'visible';
                  }, $values);
                  $taxonomy_terms[$column_name] = array_unique($mapped_terms);
               } elseif ($field_key === 'product_type') {
                  $allowed_type = [
                     'simple'       => 'simple',
                     'Simple'       => 'simple',
                     'grouped'      => 'grouped',
                     'Grouped'      => 'grouped',
                     'external'     => 'external',
                     'External'     => 'External',
                     'variable'     => 'variable',
                     'Variable'     => 'variable',
                  ];
                  $values = [trim($column_value)];
                  $mapped_terms = array_map(function ($value) use ($allowed_type) {
                     $clean = strtolower(sanitize_text_field($value));
                     return $allowed_type[$clean] ?? 'simple';
                  }, $values);
                  $taxonomy_terms[$column_name] = array_unique($mapped_terms);
                  $product_fields['_product_type'] = reset($mapped_terms);
               } else {
                  $terms = array_map('sanitize_text_field', explode(',', $column_value));
                  $taxonomy_terms[$column_name] = $terms;
               }
               break;
            case 'yoast_focus_Keyword':
               $yoast_fields['_yoast_wpseo_focuskw'] = sanitize_text_field($column_value);
               break;
            case 'yoast_meta_description':
               $yoast_fields['_yoast_wpseo_metadesc'] = sanitize_textarea_field($column_value);
               break;
            case 'yoast_cornerstone_content':
               $yoast_fields['_yoast_wpseo_is_cornerstone'] = intval($column_value);
               break;
            case 'yoast_show_content_in_search_results':
               $yoast_fields['_yoast_wpseo_meta-robots-noindex'] =
                  intval($column_value);
               break;
            case 'yoast_search_engines_follow_links':
               $yoast_fields['_yoast_wpseo_meta-robots-nofollow'] =
                  intval($column_value);
               break;
            case 'yoast_meta_robots_advanced':
               $yoast_fields['_yoast_wpseo_meta-robots-adv'] = sanitize_text_field($column_value);
               break;
            case 'yoast_breadcrumb_title':
               $yoast_fields['_yoast_wpseo_bctitle'] = sanitize_text_field($column_value);
               break;
            case 'yoast_canonical_url':
               $yoast_fields['_yoast_wpseo_canonical'] = esc_url_raw($column_value);
               break;
            case 'yoast_schema_page_type':
               $yoast_fields['_yoast_wpseo_schema_page_type'] = sanitize_text_field($column_value);
               break;
            case 'yoast_schema_article_type':
               $yoast_fields['_yoast_wpseo_schema_article_type'] = sanitize_text_field($column_value);
               break;
            case 'product_image_gallery':
               $product_image_gallery = sanitize_text_field($column_value);
               break;
            case 'product_sku':
               $product_fields['_sku'] = sanitize_text_field(trim($column_value));
               break;
            case 'product_virtual':
               $product_fields['_virtual'] = intval($column_value);
               break;
            case 'product_downloadable':
               $product_fields['_downloadable'] = intval($column_value);
               break;
            case 'product_stock':
               $product_fields['_stock'] = intval($column_value);
               break;
            case 'product_stock_status':
               $product_fields['_stock_status'] = sanitize_text_field($column_value);
               $allowed_stock_statuses = [
                  'instock'      => 'instock',
                  'in stock'     => 'instock',
                  'outofstock'   => 'outofstock',
                  'out of stock' => 'outofstock',
                  'onbackorder'  => 'onbackorder',
                  'on backorder' => 'onbackorder',
               ];
               $column_value_clean = strtolower(trim(sanitize_text_field($column_value)));
               $product_fields['_stock_status'] = isset($allowed_stock_statuses[$column_value_clean]) ? $allowed_stock_statuses[$column_value_clean] : 'instock';
               break;
            case 'product_manage_stock':
               $product_fields['_manage_stock'] = intval($column_value);
               break;
            case 'product_allow_backorders':
               $allowed_backorders = ['no', 'notify', 'yes'];
               $product_fields['_backorders'] = in_array($column_value, $allowed_backorders) ? strtolower(sanitize_text_field($column_value)) : 'no';
               break;
            case 'product_low_stock_amount':
               $product_fields['_low_stock_amount'] = intval($column_value);
               break;
            case 'product_sold_individually':
               $product_fields['_sold_individually'] = intval($column_value);
               break;
            case 'product_tax_status':
               $allowed_tax_statuses = ['taxable', 'shipping', 'none'];
               $product_fields['_tax_status'] = in_array($column_value, $allowed_tax_statuses) ? strtolower(sanitize_text_field($column_value)) : 'taxable';
               break;
            case 'product_tax_class':
               $allowed_tax_classes = [
                  'standard'     => 'standard',
                  'reduced-rate' => 'reduced-rate',
                  'reduced rate' => 'reduced-rate',
                  'zero-rate'    => 'zero-rate',
                  'zero rate'    => 'zero-rate',
               ];
               $column_value_clean = strtolower(sanitize_text_field($column_value));
               $product_fields['_tax_class'] = isset($allowed_tax_classes[$column_value_clean]) ? $allowed_tax_classes[$column_value_clean] : 'standard';
               break;
            case 'product_sale_date_price_start':
               $product_fields['_sale_price_dates_from'] = !empty($column_value)
                  ? strtotime(self::cipfa_set_date_based_on_formats($column_value))
                  : null;
               break;
            case 'product_sale_date_price_end':
               $product_fields['_sale_price_dates_to'] = !empty($column_value)
                  ? strtotime(self::cipfa_set_date_based_on_formats($column_value))
                  : null;
               break;
            case 'product_weight':
               $product_fields['_weight'] = !empty($column_value) ? floatval($column_value) : null;
               break;
            case 'product_length':
               $product_fields['_length'] = !empty($column_value) ? floatval($column_value) : null;
               break;
            case 'product_width':
               $product_fields['_width'] = !empty($column_value) ? floatval($column_value) : null;
               break;
            case 'product_height':
               $product_fields['_height'] = !empty($column_value) ? floatval($column_value) : null;
               break;
            case 'product_enable_reviews':
               $product_fields['_enable_reviews'] = intval($column_value);
               break;
            case 'product_purchase_note':
               $product_fields['_purchase_note'] = sanitize_text_field($column_value);
               break;
            case 'product_sale_price':
               $product_fields['_sale_price'] = !empty($column_value) ? floatval($column_value) : null;
               break;
            case 'product_regular_price':
               $product_fields['_regular_price'] = !empty($column_value) ? floatval($column_value) : null;
               break;
            case 'product_upsell_ids':
               $product_fields['_upsell_ids'] = array_map('intval', array_map('trim', explode(',', $column_value)));
               break;
            case 'product_crosssell_ids':
               $product_fields['_crosssell_ids'] = array_filter(
                  array_map('intval', array_map('trim', explode(',', $column_value)))
               );
               break;
            case 'product_barcode':
               $product_fields['_global_unique_id'] = sanitize_text_field($column_value);
               break;
            case 'product_download_file_name':
               $product_fields['download_file_name'] = array_map('sanitize_text_field', explode(',', $column_value));
               break;
            case 'product_download_file_url':
               $product_fields['download_file_url'] = array_map('esc_url', explode(',', $column_value));
               break;
            case 'product_download_limit':
               $product_fields['_download_limit'] = intval($column_value);
               break;
            case 'product_download_expiry':
               $product_fields['_download_expiry'] = intval($column_value);
               break;
            case 'product_wpcom_is_markdown':
               $product_fields['_wpcom_is_markdown'] = intval($column_value);
               break;
            case 'product_' . $attribute . '_attribute';
               $product_attribute_terms = array_map(
                  function ($term) {
                     return sanitize_text_field(trim($term));
                  },
                  explode(
                     '|',
                     $column_value
                  )
               );
               if (!empty($product_attribute_terms)) {
                  $product_fields["product_attribute_{$attribute}"] = [
                     'name' => self::cipfa_clean_attribute_name($attribute),
                     'value' => $product_attribute_terms
                  ];
               }
               break;
            case "product_{$attribute}_visible";
               $product_fields["product_" . self::cipfa_clean_attribute_key($attribute) . "_visible"] = intval($column_value);
               break;
            case 'product_' . $attribute . '_global';
               $product_fields["product_" . self::cipfa_clean_attribute_key($attribute) . "_global"] = intval($column_value);
               break;
            default:
               if (strpos($field_key, 'field_') !== false) {
                  $sanitized_data = CIPFA_ACFSanitizer::cipfa_sanitize_acf_data(array($field_key => $column_value));
                  if (is_array($sanitized_data) && isset($sanitized_data[$field_key])) {
                     $acf_fields[$field_key] = $sanitized_data[$field_key];
                  } else {
                     $acf_fields[$field_key] = $sanitized_data;
                  }
               }
               break;
         }
      }

      $post_data['product_fields'] = $product_fields;
      $post_data['product_image_gallery'] = $product_image_gallery;
      $post_data['acf_fields'] = $acf_fields;
      $post_data['post_tags'] = $post_tags;
      $post_data['post_category'] = $post_category;
      if (!empty($post_content)) {
         $post_data['post_content'] = $post_content;
      }
      if (!empty($post_excerpt)) {
         $post_data['post_excerpt'] = $post_excerpt;
      }
      $post_data['post_featured_image'] = $post_featured_image;
      $post_data['csv_post_id'] = $csv_post_id;
      if (!empty($taxonomy_terms)) {
         $post_data['taxonomy_terms'] = $taxonomy_terms;
      }
      $post_data['yoast_fields'] = $yoast_fields;

      return $post_data;
   }

   // Extract product's attribute Name
   public static function cipfa_extract_attribute($column_name)
   {

      $pattern = "/^product_(.*?)_(attribute|visible|global|variation)$/";

      if (preg_match(
         $pattern,
         $column_name,
         $matches
      )) {
         return $matches[1];
      }

      return '';
   }

   // Clean attribute names
   public static function cipfa_clean_attribute_name($attribute)
   {
      // Replace only underscores with spaces
      $attribute = str_replace('_', ' ', $attribute);

      // Remove all other special characters except spaces
      $attribute = preg_replace('/[^a-zA-Z0-9 ]/', '', $attribute);

      // Replace multiple spaces with a single space and trim
      return trim(preg_replace('/\s+/', ' ', $attribute));
   }

   // Clean attribute key
   public static function cipfa_clean_attribute_key($attribute)
   {
      return str_replace('_', '-', strtolower($attribute));
   }

   // Validate Post Parent
   private static function cipfa_is_valid_post_parent($post_parent)
   {
      return is_numeric($post_parent) && get_post($post_parent) && get_post_type($post_parent) == 'page';
   }

   // Validate Page Template
   private static function cipfa_is_valid_page_template($page_template_label)
   {
      $available_templates = wp_get_theme()->get_page_templates();
      $template_key = array_search($page_template_label, $available_templates);
      return $template_key !== false ? $template_key : false;
   }

   //Handle Post Author
   private static function cipfa_handle_post_author($post_author, $post_author_email)
   {
      if (!empty($post_author)) {
         // Find user by ID, username, or email
         if (is_numeric($post_author)) {
            $user = get_user_by('ID', $post_author);
         } else {
            $user = get_user_by('login', $post_author);
            if (!$user) {
               $user = get_user_by('email', $post_author);
            }
         }

         if ($user) {
            return $user->ID;
         } else {
            // Create new user if not found
            $username = sanitize_user($post_author);
            $email = sanitize_email($post_author_email);
            $author_password = !empty($author_password) ? $author_password : wp_generate_password();

            $userdata = array(
               'user_login' => $username,
               'user_pass' => $author_password,
               'user_email' => $email,
               'role' => 'author'
            );

            $user_id = wp_insert_user($userdata);

            if (!is_wp_error($user_id)) {
               return $user_id;
            } else {
               $current_user = wp_get_current_user();
               CIPFA_Logs::cipfa_Log('Error creating a new author with email ' . $post_author_email . ': ' . $user_id->get_error_message() . ' Using current author ' . $current_user->user_login);
               return get_current_user_id(); // Use current user if user creation fails
            }
         }
      } else {
         return get_current_user_id();
      }
   }

   //Insert or Update Post
   private static function cipfa_insert_or_update_post($post_data, $edit_existing, $csv_post_id)
   {
      if (
         isset($post_data['post_type']) &&
         $post_data['post_type'] === 'product' &&
         isset($post_data['taxonomy_terms']['product_type']) &&
         !in_array(
            'simple',
            $post_data['taxonomy_terms']['product_type']
         )
      ) {
         return 0; // Don't proceed with insertion or update
      }

      // Default valid post statuses and status map
      $valid_statuses = array(
         'publish', 'future', 'draft', 'pending', 'private', 'trash', 'auto-draft', 'inherit'
      );
      $status_map = array(
         'published' => 'publish',
         'scheduled' => 'future',
         'schedule'  => 'future',
         'trashed'   => 'trash',
         1           => 'publish',
         0           => 'draft',
         -1          => 'pending',
         2           => 'future',
         3           => 'private'
      );

      // "post_status" is not provided, use the existing post's status & log if post ID is invalid
      if ($edit_existing && !empty($csv_post_id)) {
         $existing_post = get_post($csv_post_id);
         // Log if the provided post ID is invalid
         if (!$existing_post) {
            CIPFA_Logs::cipfa_Log('Failed to find post with ID ' . $csv_post_id . '. Update aborted.');
            return 0;
         }

         if ($existing_post->post_type === 'attachment') {
            CIPFA_Logs::cipfa_Log('Post ID ' . $csv_post_id . ' is an attachment. Update aborted.');
            return 0;
         }
         //use the existing post's status
         if (
            !isset($post_data['post_status']) || $post_data['post_status'] === ''
         ) {
            $post_data['post_status'] = $existing_post->post_status;
         }
      } else {
         // For new posts, default to 'publish'
         if (!isset($post_data['post_status']) || $post_data['post_status'] === '') {
            $post_data['post_status'] = 'publish';
         }
      }

      // Normalize post status to lowercase and map if needed
      $post_data['post_status'] = strtolower($post_data['post_status']);
      if (array_key_exists($post_data['post_status'], $status_map)) {
         $post_data['post_status'] = $status_map[$post_data['post_status']];
      }

      // Validate post status before inserting/updating
      if (!in_array($post_data['post_status'], $valid_statuses)) {
         CIPFA_Logs::cipfa_Log('Invalid post status: ' . $post_data['post_status'] . '. Post ' . $post_data['post_title'] . ' is not inserted or updated.');
         return 0; // Don't proceed with insertion or update
      }

      // Update an existing post
      if ($edit_existing && !empty($csv_post_id)) {
         $existing_post = get_post($csv_post_id);
         if (
            $existing_post && in_array($existing_post->post_status, $valid_statuses)
         ) {
            $post_data['ID'] = $csv_post_id;
            $post_id = wp_update_post($post_data);

            // Check for errors while updating the post
            if (is_wp_error($post_id)) {
               CIPFA_Logs::cipfa_Log('Failed to update post with ID ' . $csv_post_id . '. Error: ' . $post_id->get_error_message());
               return 0;
            }
            return $post_id;
         }
      }

      // Insert a new post
      $post_id = wp_insert_post($post_data);

      // Check if post insertion failed
      if (is_wp_error($post_id)) {
         CIPFA_Logs::cipfa_Log('Failed to insert post with title ' . $post_data['post_title'] . '. Error: ' . $post_id->get_error_message());
         return 0;
      }
      return $post_id;
   }

   //Set Featured Image
   private static function cipfa_set_featured_image($post_id, $post_featured_image)
   {
      // Image URL validate
      $post_featured_image = self::cipfa_validate_and_log_url($post_featured_image);

      if ($post_featured_image === false) {
         return false;
      }

      // Image process
      $attachment_id = self::cipfa_process_media($post_featured_image, $post_id, 'post-id');

      if (!$attachment_id) {
         return;
      }

      // Check if the post already has the same featured image set
      $current_thumbnail_id = get_post_thumbnail_id($post_id);
      if ($current_thumbnail_id == $attachment_id) {
         return;
      }

      // Set the featured image for the post
      set_post_thumbnail($post_id, $attachment_id);
   }

   //Find existing attachments by file path
   public static function cipfa_find_existing_attachments($normalized_url)
   {
      $option_key = 'cipfa_duplicate_media';

      // Retrieve the stored duplicate from the options table
      $stored_duplicate_media = get_option($option_key, []);

      // Check if the option value is initialized
      if (empty($stored_duplicate_media)) {
         return null;
      }

      // check the value is set and not empty for the given normalized URL
      if (isset($stored_duplicate_media[$normalized_url]) && !empty($stored_duplicate_media[$normalized_url])) {
         return $stored_duplicate_media[$normalized_url];
      }
      return null;
   }

   //Register Media in media library
   public static function cipfa_register_media_in_media_library($normalized_url)
   {
      $option_key = 'cipfa_duplicate_media';

      $media_path = wp_normalize_path(str_replace(get_site_url() . '/', ABSPATH, $normalized_url));
      $stored_duplicate_media = get_option($option_key, []);

      // Get file info
      $filetype = wp_check_filetype($media_path);
      $filename = basename($media_path);

      // Copy file to the uploads directory
      $upload_dir = wp_upload_dir();
      $destination_path = $upload_dir['path'] . '/' . $filename;

      if (!file_exists($destination_path)) {
         copy($media_path, $destination_path);
      }

      // Prepare attachment data
      $attachment = [
         'guid'           => $upload_dir['url'] . '/' . $filename,
         'post_mime_type' => $filetype['type'],
         'post_title'     => sanitize_file_name($filename),
         'post_content'   => '',
         'post_status'    => 'inherit',
      ];

      // Insert attachment into the database
      $attachment_id = wp_insert_attachment($attachment, $destination_path);

      // Update the option with the new attachment ID
      if (!isset($stored_duplicate_media[$normalized_url]) || empty($stored_duplicate_media[$normalized_url])) {
         $stored_duplicate_media[$normalized_url] = $attachment_id;
         update_option($option_key, $stored_duplicate_media, false);
      }

      // Generate attachment metadata and update
      if (!is_wp_error($attachment_id)) {
         require_once ABSPATH . 'wp-admin/includes/image.php';
         $attachment_data = wp_generate_attachment_metadata($attachment_id, $destination_path);
         wp_update_attachment_metadata($attachment_id, $attachment_data);
      }
      return $attachment_id;
   }

   //Extract image URLs from post content & post excerpt
   private static function cipfa_extract_image_sources_from_post_field($post_id, $content)
   {
      $image_urls = [];
      $pattern = '/<img[^>]+src\s*=\s*["\']\s*([^"\']+?)\s*["\']/i';

      if (preg_match_all($pattern, $content, $matches)) {
         foreach ($matches[1] as $src) {
            // Image URL validation
            $src = CIPFA_PostProcessor::cipfa_validate_and_log_url($src);

            if ($src === false) {
               continue;
            }

            // Convert URL to file path
            $image_path = wp_normalize_path(str_replace(get_site_url() . '/', ABSPATH, $src));

            // Check if the file exists
            if (file_exists($image_path)) {
               $attachment_id = attachment_url_to_postid($src);
               if (!$attachment_id) {
                  $image_urls[] = $src;
               }
            } else {
               CIPFA_Logs::cipfa_Log('Image not found or invalid path: ' . $src . " for post " . $post_id);
            }
         }
      }

      return $image_urls;
   }

   //Replace images in post content & post excerpt
   private static function cipfa_handle_images_in_post_field($post_id, $field, $content)
   {
      // Extract image URLs
      $image_urls = self::cipfa_extract_image_sources_from_post_field($post_id, $content);

      if (empty($image_urls)) {
         return $content;
      }

      foreach ($image_urls as $image_url) {
         $normalized_url = esc_url_raw($image_url);

         $attachment_id = self::cipfa_find_existing_attachments($normalized_url);

         if (!$attachment_id) {
            $attachment_id = self::cipfa_register_media_in_media_library($normalized_url);

            if (!$attachment_id) {
               CIPFA_Logs::cipfa_Log('Failed to register image in the Media Library: ' . $normalized_url . " for post " . $post_id);
               continue;
            }
         }

         $new_url = wp_get_attachment_url($attachment_id);
         $content = str_replace($image_url, $new_url, $content);
      }

      // Update the post
      wp_update_post([
         'ID' => $post_id,
         $field => $content,
      ]);

      return $content;
   }

   //Set Categories
   private static function cipfa_set_categories($post_id, $post_category)
   {
      if (!empty($post_category)) {
         $category_ids = array();

         foreach ($post_category as $category_path) {
            // Decode HTML entities (e.g., &gt; to >) and trim whitespace
            $category_path = html_entity_decode(trim($category_path));
            // Split by ">" to handle parent-child hierarchy
            $categories = explode('>', $category_path);
            $parent_id = 0;

            foreach ($categories as $category_name) {
               $category_name = trim($category_name);

               // Check if the category exists under the current parent
               $term = term_exists($category_name, 'category', $parent_id);

               if ($term !== 0 && $term !== null) {
                  // Category exists, set parent_id for the next level in hierarchy
                  $parent_id = $term['term_id'];
               } else {
                  // Create category under the current parent
                  $new_term = wp_insert_term($category_name, 'category', array('parent' => $parent_id));

                  if (!is_wp_error($new_term)) {
                     $parent_id = $new_term['term_id'];
                  }
               }
            }

            // Add the final category ID to the list
            if (!empty($parent_id)) {
               $category_ids[] = $parent_id;
            }
         }

         // Assign categories to the post
         if (!empty($category_ids)) {
            wp_set_post_terms($post_id, $category_ids, 'category');
         }
      }
   }

   //Set Custom texonomy
   private static function cipfa_set_custom_taxonomy_terms($post_id, $taxonomy_terms)
   {
      if (!empty($taxonomy_terms)) {
         foreach ($taxonomy_terms as $taxonomy => $terms) {
            $term_ids = array();
            foreach ($terms as $term_path) {
               $term_path = sanitize_text_field(html_entity_decode(trim($term_path)));
               // Split by ">" to handle parent-child hierarchy
               $hierarchical_terms = explode('>', $term_path);
               $parent_id = 0;
               foreach ($hierarchical_terms as $term_name) {
                  $term_name = trim($term_name);
                  if (empty($term_name)) {
                     continue;
                  }
                  $existing_term = term_exists($term_name, $taxonomy, $parent_id);

                  if ($existing_term && !is_wp_error($existing_term)) {
                     $parent_id = (int)$existing_term['term_id'];
                  } else {
                     $new_term = wp_insert_term($term_name, $taxonomy, array('parent' => $parent_id));
                     if (!is_wp_error($new_term)) {
                        $parent_id = (int)$new_term['term_id'];
                     }
                  }
               }
               if (!empty($parent_id)) {
                  $term_ids[] = $parent_id;
               }
            }
            if (!empty($term_ids)) {
               wp_set_post_terms($post_id, $term_ids, $taxonomy);
            }
         }
      }
   }

   //Set Tags
   private static function cipfa_set_tags($post_id, $post_tags)
   {
      if (!empty($post_tags)) {
         wp_set_post_tags($post_id, $post_tags);
      }
   }

   // Media URL validate
   public static function cipfa_validate_and_log_url($url)
   {
      // Sanitize the URL
      $url = esc_url_raw($url);

      // Return false if no media URL is provided
      if (empty($url)) {
         return false;
      }

      // Remove the protocol from the URLs
      $acf_image_no_protocol = preg_replace('#^https?://#', '', $url);
      $site_url_no_protocol = preg_replace('#^https?://#', '', get_site_url());

      // Extract the domain parts from both URLs
      $post_image_domain = wp_parse_url('http://' . $acf_image_no_protocol, PHP_URL_HOST);
      $site_url_domain = wp_parse_url('http://' . $site_url_no_protocol, PHP_URL_HOST);

      // Check if the domains match but protocols are different
      if (
         $post_image_domain === $site_url_domain
      ) {
         $image_protocol = wp_parse_url($url, PHP_URL_SCHEME);
         $site_protocol = wp_parse_url(get_site_url(), PHP_URL_SCHEME);

         if ($image_protocol !== $site_protocol) {
            // Correct the protocol
            $corrected_url = str_replace("$image_protocol://", "$site_protocol://", $url);

            // Log protocol mismatch only once
            if (get_option('cipfa_protocol_mismatch_log_shown', false) === false) {
               CIPFA_Logs::cipfa_Log(
                  "Protocol mismatch detected and corrected in media URLs"
               );
               update_option('cipfa_protocol_mismatch_log_shown', true);
            }
            return $corrected_url; // Return the corrected URL
         }
      }

      // Skip external URLs
      if (stripos($url, get_site_url()) === false) {
         if (get_option('cipfa_external_url_log_shown', false) === false) {
            CIPFA_Logs::cipfa_Log(
               "External URLs not supported, won't upload to media library"
            );
            update_option('cipfa_external_url_log_shown', true);
         }
         return false;
      }

      return $url; // Return the original valid URL
   }

   // Media process
   public static function cipfa_process_media($media_url, $context = null, $context_type = 'post-id')
   {
      // Try to get the attachment ID from the URL
      $attachment_id = attachment_url_to_postid($media_url);

      if (!$attachment_id) {
         // Convert the URL to a local file path
         $media_path = wp_normalize_path(str_replace(get_site_url() . '/', ABSPATH, $media_url));

         // Check if the file exists
         if (file_exists($media_path)) {
            // Find existing attachments
            $existing_attachment = self::cipfa_find_existing_attachments($media_url);

            if ($existing_attachment) {
               return $existing_attachment;
            }

            // Register the media in the Media Library
            $attachment_id = self::cipfa_register_media_in_media_library($media_url);

            if (!$attachment_id) {
               $context_label = $context_type === 'field-key' ? "field-key: $context" : "post-id: $context";
               CIPFA_Logs::cipfa_Log(
                  "Failed to register media in the Media Library for $context_label. URL: $media_url"
               );
               return false;
            }
         } else {
            $context_label = $context_type === 'field-key' ? "field-key: $context" : "post-id: $context";
            CIPFA_Logs::cipfa_Log(
               "Media file not found for $context_label. URL: $media_url"
            );
            return false;
         }
      }

      return $attachment_id;
   }

   //Set Product Gallery images
   private static function cipfa_set_product_image_gallery($post_id, $product_image_gallery)
   {
      // Convert the comma-separated string into an array
      $gallery_images = array_map('trim', explode(',', $product_image_gallery));

      $processed_images = [];

      foreach ($gallery_images as $image_url) {

         // Image URL validate
         $image_url = CIPFA_PostProcessor::cipfa_validate_and_log_url($image_url);

         if ($image_url === false) {
            continue;
         }

         // Image process
         $attachment_id = self::cipfa_process_media($image_url, $post_id, 'post-id');

         if (!$attachment_id) {
            continue;
         }

         // Add the attachment ID to the processed images array
         $processed_images[] = $attachment_id;
      }

      // Return a comma-separated string of attachment IDs
      if (!empty($processed_images)) {
         return implode(',', $processed_images);
      }

      // Return an empty string if no valid images were processed
      return '';
   }

   // Set Date
   private static function cipfa_set_date_based_on_formats($column_value)
   {
      $date_formats = [
         'm/d/Y', 'd/m/Y', 'd-M-y', 'Y-m-d'
      ];
      $date_obj = false;

      foreach ($date_formats as $format) {
         $date_obj = DateTime::createFromFormat($format, $column_value);
         $errors = DateTime::getLastErrors();

         if (
            $date_obj !== false && $errors['warning_count'] === 0 && $errors['error_count'] === 0
         ) {
            break;
         }

         $date_obj = false; // Reset if parsing failed
      }

      if ($date_obj !== false) {
         return $date_obj->format('Y-m-d H:i:s');
      } else {
         // Return current date and time if parsing fails
         return (new DateTime())->format('Y-m-d H:i:s');
      }
   }

   // Generating unique slug for terms
   private static function cipfa_generate_unique_slug($term, $taxonomy)
   {
      $base_slug = sanitize_title($term);
      $slug      = $base_slug;
      $counter   = 1;

      while (true) {
         $existing_term = get_term_by('slug', $slug, $taxonomy);
         if (!$existing_term) {
            break;
         }
         // term exists
         if ($existing_term && $existing_term->name === $term) {
            break;
         }
         // counter to the base slug
         $slug = $base_slug . '-' . $counter;
         $counter++;
      }
      return $slug;
   }

   // Set Product Attributes
   private static function cipfa_set_product_attributes($post_id, &$product_fields)
   {
      // Ensure _product_attributes key exists
      if (!isset($product_fields['_product_attributes']) || !is_array($product_fields['_product_attributes'])) {
         $product_fields['_product_attributes'] = [];
      }

      foreach ($product_fields as $key => $attribute_data) {
         if (strpos($key, 'product_attribute_') === 0 && !empty($attribute_data)) {
            $attribute_label       = $attribute_data['name'];
            $product_attribute_name = sanitize_title($attribute_label);
            $product_attribute_terms = array_filter($attribute_data['value']);

            // Skip attributes without terms
            if (empty($product_attribute_terms)) {
               continue;
            }

            $taxonomy_name = "pa_{$product_attribute_name}";

            $is_visible = isset($product_fields["product_{$product_attribute_name}_visible"]) ? $product_fields["product_{$product_attribute_name}_visible"] : 1;
            $is_global  = isset($product_fields["product_{$product_attribute_name}_global"]) ? $product_fields["product_{$product_attribute_name}_global"] : 1;

            if ($is_global) {
               // Handle global attributes
               $existing_attributes = wc_get_attribute_taxonomies();
               $attribute_exists    = false;

               foreach ($existing_attributes as $attribute) {
                  if ($attribute->attribute_name === $product_attribute_name) {
                     $attribute_exists = true;
                     break;
                  }
               }

               // Register attribute if it doesn't exist
               if (!$attribute_exists) {
                  $attribute_data_args = [
                     'name'         => $attribute_label,
                     'slug'         => $product_attribute_name,
                     'type'         => 'select',
                     'order_by'     => 'menu_order',
                     'has_archives' => false,
                  ];

                  $attribute_result = wc_create_attribute($attribute_data_args);

                  if (!is_wp_error($attribute_result)) {
                     delete_transient('wc_attribute_taxonomies');
                     register_taxonomy($taxonomy_name, 'product', [
                        'label'         => ucfirst($attribute_label),
                        'hierarchical'  => false,
                        'show_ui'       => false,
                     ]);
                  }
               }

               // Add terms to the taxonomy
               $term_slugs = [];
               foreach ($product_attribute_terms as $term) {
                  $term_data = get_term_by('name', $term, $taxonomy_name);
                  if (!$term_data) {
                     $unique_slug  =
                        self::cipfa_generate_unique_slug($term, $taxonomy_name);
                     $inserted_term = wp_insert_term(
                        $term,
                        $taxonomy_name,
                        ['slug' => $unique_slug]
                     );
                     if (!is_wp_error($inserted_term)) {
                        $term_slugs[] = $unique_slug;
                     }
                  } else {
                     $term_slugs[] = $term_data->slug;
                  }
               }

               // Set product terms using term slugs
               wp_set_object_terms($post_id, $term_slugs, $taxonomy_name);

               // Prepare global attribute data
               $product_fields['_product_attributes'][$taxonomy_name] = [
                  'name'         => $taxonomy_name,
                  'value'        => implode('|', $term_slugs),
                  'position'     => 0,
                  'is_visible'   => $is_visible,
                  'is_variation' => 0,
                  'is_taxonomy'  => 1,
               ];
            } else {
               // Handle non-global attributes
               $product_fields['_product_attributes'][$product_attribute_name] = [
                  'name'         => $attribute_label,
                  'value'        => implode('|', $product_attribute_terms),
                  'position'     => 0,
                  'is_visible'   => $is_visible,
                  'is_variation' => 0,
                  'is_taxonomy'  => 0,
               ];
            }
         }
      }
      update_post_meta($post_id, '_product_attributes', $product_fields['_product_attributes']);
   }

   //Set Product Download file name and URL
   private static function cipfa_set_product_downloadable_files(&$product_fields)
   {
      $file_names = $product_fields['download_file_name'] ?? [];
      $file_urls  = $product_fields['download_file_url'];
      $product_downloadable_files = [];

      // Get the Register class instance for approved directories
      $register = new \Automattic\WooCommerce\Internal\ProductDownloads\ApprovedDirectories\Register();

      // Get current site host
      $site_url  = home_url();
      $site_host = wp_parse_url($site_url, PHP_URL_HOST) ?: '';

      foreach ($file_urls as $index => $url) {
         if (!empty($url)) {
            // Check if the URL is within an approved directory
            $is_approved = $register->is_valid_path($url);

            // If the URL is not approved, check the domain
            if (!$is_approved) {
               $url_parts = wp_parse_url($url);
               $url_host = isset($url_parts['host']) ? $url_parts['host'] : '';

               // Only process upload if the URL is on the same domain
               if ($url_host === $site_host) {
                  $attachment_id = CIPFA_PostProcessor::cipfa_process_media($url, 'download_file_url', 'field-key');
                  if (!$attachment_id) {
                     continue;
                  }

                  // Get the URL
                  $url = wp_get_attachment_url($attachment_id);

                  // Extract the directory
                  $new_dir = trailingslashit(dirname($url));

                  // If this directory is not already approved, add it
                  if (!$register->approved_directory_exists($new_dir)) {
                     $register->add_approved_directory($new_dir, true);
                  }
               }
            }

            // Use the provided file name or derive it from the URL if not provided
            $name = !empty($file_names[$index])
               ? $file_names[$index]
               : basename(wp_parse_url($url, PHP_URL_PATH) ?? '');

            $unique_key = 'download_' . ($index + 1);

            $product_downloadable_files[$unique_key] = [
               'name' => $name,
               'file' => $url,
            ];
         }
      }

      if (!empty($product_downloadable_files)) {
         $product_fields['_downloadable_files'] = $product_downloadable_files;
      }
   }

   // Determine active price
   private static function cipfa_update_product_price(&$product_fields)
   {
      if (isset($product_fields['_regular_price'])) {
         if (isset($product_fields['_sale_price']) && $product_fields['_sale_price'] < $product_fields['_regular_price']) {
            $product_fields['_price'] = $product_fields['_sale_price'];
         } else {
            $product_fields['_price'] = $product_fields['_regular_price'];
            $product_fields['_sale_price'] = '';
         }
      }
   }

   //Update ACF Fields
   private static function cipfa_update_acf_fields($post_id, $acf_fields)
   {
      if (!empty($acf_fields)) {
         foreach ($acf_fields as $field_key => $field_value) {
            if (!empty($field_value)) {
               update_field($field_key, $field_value, $post_id);
            }
         }
      }
   }

   //Update yoast Fields
   private static function cipfa_update_yoast_fields($post_id, $yoast_fields)
   {
      if (!empty($yoast_fields)) {
         foreach ($yoast_fields as $field_key => $field_value) {
            if (!empty($field_value)) {
               update_post_meta($post_id, $field_key, $field_value);
            }
         }
      }
   }

   //Update product Fields
   private static function cipfa_update_product_fields($post_id, $product_fields)
   {
      if (!empty($product_fields)) {
         foreach ($product_fields as $field_key => $field_value) {
            if (!empty($field_value)) {
               update_post_meta($post_id, $field_key, $field_value);
            }
         }
      }
   }
}
