<?php

/**
 * @package CsvImporterPlusforACF
 * Mapping Form
 */

?>

<?php

// make sure we don't expose any info if called directly
if (!defined('ABSPATH')) exit; // Exit if accessed directly

use League\Csv\Reader;
use League\Csv\Exception;
use League\Csv\SyntaxError;
use CIPFA\base\CIPFA_BaseController;
use CIPFA\base\CIPFA_PostProcessor;

// Verify nonce before processing the CSV file upload
if (isset($_POST['cipfa_nonce_field'])) {
   $nonce = sanitize_text_field(wp_unslash($_POST['cipfa_nonce_field']));
   if (!wp_verify_nonce($nonce, 'cipfa_nonce')) {
      wp_die(esc_html__('Nonce verification failed. Please try again.', 'csv-importer-plus-for-acf'));
   }
} else {
   wp_die(esc_html__('Nonce field is missing.', 'csv-importer-plus-for-acf'));
}

if (isset($_FILES['csv_file']) && isset($_FILES['csv_file']['tmp_name'])) {

   // Handle the file upload
   $upload_overrides = array('test_form' => false);
   $movefile = wp_handle_upload($_FILES['csv_file'], $upload_overrides);

   if ($movefile && !isset($movefile['error'])) {
      // File uploaded successfully
      $target_file = $movefile['file'];
      update_option('cipfa_csv_file_path', $target_file);
   } else {
      // Handle upload error
      /* translators: %s is the upload error message */
      echo '<div class="error"><p>' . sprintf(esc_html__('Error uploading file: %s', 'csv-importer-plus-for-acf'), esc_html($movefile['error'])) . '</p></div>';
      return;
   }
} else {
   echo '<div class="error"><p>' . esc_html__('No CSV file uploaded.', 'csv-importer-plus-for-acf') . '</p></div>';
   return;
}

try {
   $csv_file = get_option('cipfa_csv_file_path');

   // Create a CSV Reader instance
   $csv = Reader::createFromPath($csv_file, 'r');

   // Set the header offset if the first row contains headers
   $csv->setHeaderOffset(0);

   // Header row
   $header_row = $csv->getHeader();
} catch (SyntaxError $e) {
   echo '<div class="mt-6 bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded relative" role="alert">';
   echo '<strong class="font-bold">' . esc_html__('Error:', 'csv-importer-plus-for-acf') . '</strong> ';
   echo '<span>' . esc_html__('The CSV file contains duplicate column names. Please ensure all column names are unique and try again.', 'csv-importer-plus-for-acf') . '</span>';
   echo '</div>';
   exit;
} catch (Exception $e) {
   echo '<div class="mt-6 bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded relative" role="alert">';
   echo '<strong class="font-bold">' . esc_html__('Error:', 'csv-importer-plus-for-acf') . '</strong> ';
   echo '<span>' . esc_html($e->getMessage()) . '</span>';
   echo '</div>';
   exit;
}
// Retrieve the filtered post types
$filtered_post_types = CIPFA_BaseController::cipfa_get_filtered_post_types();
// Get selected post type
$post_type = isset($_POST['post_type']) ? sanitize_text_field(wp_unslash($_POST['post_type'])) : 'post';
if (isset($filtered_post_types[$post_type])) {
   // Get taxonomies for the validated post type
   $taxonomies = get_object_taxonomies($post_type, 'objects');
   // Filter for custom taxonomies only
   $custom_taxonomies = array_filter($taxonomies, function ($taxonomy) {
      return $taxonomy->_builtin === false;
   });
}
// Get chunck size
$chunk_size = isset($_POST['chunk_size']) ? intval($_POST['chunk_size']) : 10;
// Get update check
$edit_existing = isset($_POST['edit_existing']) ? ($_POST['edit_existing'] == '1') : false;
// Get cache check
$flush_cache = isset($_POST['flush_cache']) ? ($_POST['flush_cache'] == '1') : false;
//check if ACF exists
if (class_exists('ACF')) {
   // Get ACF field groups associated with the selected post type
   $acf_field_groups = acf_get_field_groups(array('post_type' => $post_type));
}
// Trim and clean all column names in the header row
$header_row = array_map('CIPFA\base\CIPFA_FileHandler::cipfa_clean_column_name', $header_row);
// Count occurrences of each column name
$column_counts = array_count_values($header_row);
// Determine unique column names
$unique_column_names = array_filter($column_counts, function ($count) {
   return $count === 1;
});
?>
<!-- Display CSV mapping form -->
<div class="max-w-3xl mx-auto mt-8 p-8 bg-white rounded-lg shadow-lg" id="importer-form-container">
   <h2 class="text-2xl mb-8 text-center font-semibold" id="id_map_text"><?php echo esc_html__('Map CSV Columns to Post Fields and ACF Fields', 'csv-importer-plus-for-acf') ?></h2>
   <form id="mapping-form" method="post" class="space-y-4">
      <input type="hidden" name="post_type" value="<?php echo esc_attr($post_type); ?>" />
      <input type="hidden" name="chunk_size" value="<?php echo esc_attr($chunk_size); ?>" />
      <input type="hidden" name="edit_existing" value="<?php echo esc_attr($edit_existing); ?>" />
      <input type="hidden" name="flush_cache" value="<?php echo esc_attr($flush_cache); ?>" />
      <?php wp_nonce_field('cipfa_nonce', 'cipfa_nonce_field'); ?>
      <input type="hidden" name="action" value="cipfa_data">
      <div class="overflow-x-auto">
         <table class="w-full divide-y divide-gray-200">
            <thead class="bg-gray-50">
               <tr>
                  <th scope="col" class="px-2 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider w-1/3">
                     <?php echo esc_html__('CSV Column', 'csv-importer-plus-for-acf') ?>
                  </th>
                  <th scope="col" class="px-6 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider">
                     <?php echo esc_html__('Field', 'csv-importer-plus-for-acf') ?>
                  </th>
               </tr>
            </thead>
            <tbody class="bg-white divide-y divide-gray-200">
               <?php foreach ($header_row as $column_name) : ?>
                  <?php
                  CIPFA\base\CIPFA_FileHandler::cipfa_clean_column_name($column_name);
                  ?>
                  <tr class="border-none">
                     <td class="px-2 py-4 whitespace-nowrap">
                        <div class="text-sm text-gray-900 px-4"><?php echo esc_html($column_name); ?></div>
                     </td>
                     <td class="px-6 py-4 whitespace-nowrap">
                        <select name="mapping[<?php echo esc_attr($column_name); ?>]" class="block w-full border-gray-200 focus:border-indigo-500 focus:ring-indigo-500 rounded-md">
                           <option value=""><?php echo esc_html__('Select Field', 'csv-importer-plus-for-acf') ?></option>
                           <?php if ($post_type != 'page' && $post_type != 'attachment') : ?>
                              <optgroup label="Post Fields">
                                 <?php
                                 if ($edit_existing) { ?>
                                    <option value="post_id" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_id');
                                                            endif; ?>><?php echo esc_html__('Post ID', 'csv-importer-plus-for-acf') ?></option>
                                 <?php } ?>
                                 <option value="post_title" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_title');
                                                            endif; ?>><?php echo esc_html__('Title', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_date" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_date');
                                                            endif; ?>><?php echo esc_html__('Published Date', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_content" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_content');
                                                               endif; ?>><?php echo esc_html__('Content', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_excerpt" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_excerpt');
                                                               endif; ?>><?php echo esc_html__('Excerpt', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="featured_image" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_featured_image');
                                                                  endif; ?>><?php echo esc_html__('Featured Image', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_category" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_categories');
                                                               endif; ?>><?php echo esc_html__('Categories', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="tags" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_tags');
                                                      endif; ?>><?php echo esc_html__('Tags', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_author" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_author_usernames');
                                                               endif; ?>><?php echo esc_html__('Author Usernames', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_author_email" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_author_emails');
                                                                     endif; ?>><?php echo esc_html__('Author Emails', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="author_password" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_author_passwords');
                                                                  endif; ?>><?php echo esc_html__('Author Passwords', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_status" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_status');
                                                               endif; ?>><?php echo esc_html__('Post Status', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_password" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_password');
                                                               endif; ?>><?php echo esc_html__('Post Password', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="comments" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_comments');
                                                            endif; ?>><?php echo esc_html__('Comments (enable/disable)', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="pings" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'post_pings');
                                                         endif; ?>><?php echo esc_html__('Pings (enable/disable)', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="menu_order" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'menu_order');
                                                            endif; ?>><?php echo esc_html__('Post\'s Order', 'csv-importer-plus-for-acf') ?></option>
                              </optgroup>
                           <?php elseif ($post_type == 'page') : ?>
                              <optgroup label="Page Fields">
                                 <?php
                                 if ($edit_existing) { ?>
                                    <option value="post_id" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_id');
                                                            endif; ?>><?php echo esc_html__('Post ID', 'csv-importer-plus-for-acf') ?></option>
                                 <?php } ?>
                                 <option value="post_title" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_title');
                                                            endif; ?>><?php echo esc_html__('Title', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_date" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_date');
                                                            endif; ?>><?php echo esc_html__('Published Date', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_content" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_content');
                                                               endif; ?>><?php echo esc_html__('Content', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_excerpt" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_excerpt');
                                                               endif; ?>><?php echo esc_html__('Excerpt', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="featured_image" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_featured_image');
                                                                  endif; ?>><?php echo esc_html__('Featured Image', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_parent" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_parent');
                                                               endif; ?>><?php echo esc_html__('Page\'s Parent', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="menu_order" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'menu_order');
                                                            endif; ?>><?php echo esc_html__('Page\'s Order', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="page_template" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_template');
                                                               endif; ?>><?php echo esc_html__('Page\'s Template', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_author" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_author_usernames');
                                                               endif; ?>><?php echo esc_html__('Author Usernames', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_author_email" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_author_email');
                                                                     endif; ?>><?php echo esc_html__('Author Emails', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="author_password" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_author_password');
                                                                  endif; ?>><?php echo esc_html__('Author Passwords', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_status" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_status');
                                                               endif; ?>><?php echo esc_html__('Page Status', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="post_password" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_password');
                                                               endif; ?>><?php echo esc_html__('Page Password', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="comments" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'page_comments');
                                                            endif; ?>><?php echo esc_html__('Comments (enable/disable)', 'csv-importer-plus-for-acf') ?></option>
                              </optgroup>
                           <?php endif; ?>

                           <!-- Custom Taxonomies -->
                           <?php if (!empty($custom_taxonomies)) : ?>
                              <optgroup label="Custom Taxonomies">
                                 <?php foreach ($custom_taxonomies as $taxonomy) : ?>
                                    <option value="<?php echo esc_attr($taxonomy->name); ?>" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, esc_html($taxonomy->name));
                                                                                             endif; ?>><?php echo esc_html($taxonomy->label); ?></option>
                                 <?php endforeach; ?>
                              </optgroup>
                           <?php endif; ?>

                           <!--  Yoast SEO fields -->
                           <?php if (defined('WPSEO_VERSION')) { ?>
                              <optgroup label="Yoast SEO Fields">
                                 <option value="yoast_focus_Keyword" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_focus_Keyword');
                                                                     endif; ?>><?php echo esc_html__('Focus Keyword', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_meta_description" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_meta_description');
                                                                        endif; ?>><?php echo esc_html__('Meta Description', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_cornerstone_content" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_cornerstone_content');
                                                                           endif; ?>><?php echo esc_html__('Cornerstone Content', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_meta_robots_advanced" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_meta_robots_advanced');
                                                                              endif; ?>><?php echo esc_html__('Meta Robots Advanced', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_show_content_in_search_results" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_show_content_in_search_results');
                                                                                       endif; ?>><?php echo esc_html__('Show Content In Search Results', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_search_engines_follow_links" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_search_engines_follow_links');
                                                                                    endif; ?>><?php echo esc_html__('Search Engines Follow Links', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_breadcrumb_title" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_breadcrumb_title');
                                                                        endif; ?>><?php echo esc_html__('Breadcrumb Title', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_canonical_url" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_canonical_url');
                                                                     endif; ?>><?php echo esc_html__('Canonical URL', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_schema_page_type" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_schema_page_type');
                                                                        endif; ?>><?php echo esc_html__('Schema Page Type', 'csv-importer-plus-for-acf') ?></option>
                                 <option value="yoast_schema_article_type" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'yoast_schema_article_type');
                                                                           endif; ?>><?php echo esc_html__('Schema Article  Type', 'csv-importer-plus-for-acf') ?></option>
                              </optgroup>
                           <?php } ?>

                           <!-- WooCommerce Fields -->
                           <?php if (class_exists('WooCommerce')) { ?>
                              <optgroup label="WooCommerce Fields">
                                 <?php
                                 if ($edit_existing) { ?>
                                    <option value="product_id" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_id');
                                                               endif; ?>><?php echo esc_html__('Product ID', 'csv-importer-plus-for-acf') ?></option>
                                 <?php } ?>
                                 <option value="product_name" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_name');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Name', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_description" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_description');
                                                                     endif; ?>>
                                    <?php echo esc_html__('Product Description', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_published" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_published');
                                                                     endif; ?>>
                                    <?php echo esc_html__('Product Published', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_date" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_date');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Date', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_cat" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_cat');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Categories', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_tag" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_tag');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Tags', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_image" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_image');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Image', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_image_gallery" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_image_gallery');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Image Gallery', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_sku" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_sku');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product SKU', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_short_description" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_short_description');
                                                                           endif; ?>>
                                    <?php echo esc_html__('Product Short Description', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_virtual" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_virtual');
                                                                  endif; ?>>
                                    <?php echo esc_html__('Product Virtual', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_downloadable" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_downloadable');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Downloadable', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_stock" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_stock');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Stock', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_stock_status" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_stock_status');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Stock Status', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_manage_stock" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_manage_stock');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Manage Stock', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_allow_backorders" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_allow_backorders');
                                                                           endif; ?>>
                                    <?php echo esc_html__('Product Allow Backorders', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_low_stock_amount" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_low_stock_amount');
                                                                           endif; ?>>
                                    <?php echo esc_html__('Product Low Stock Amount', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_sold_individually" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_sold_individually');
                                                                           endif; ?>>
                                    <?php echo esc_html__('Product Sold Individually', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_tax_status" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_tax_status');
                                                                     endif; ?>>
                                    <?php echo esc_html__('Product Tax Status', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_tax_class" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_tax_class');
                                                                     endif; ?>>
                                    <?php echo esc_html__('Product Tax Class', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_sale_date_price_start" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_sale_date_price_start');
                                                                                 endif; ?>>
                                    <?php echo esc_html__('Sale date price start', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_sale_date_price_end" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_sale_date_price_end');
                                                                              endif; ?>>
                                    <?php echo esc_html__('Sale date price end', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_weight" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_weight');
                                                                  endif; ?>>
                                    <?php echo esc_html__('Product Weight', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_length" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_length');
                                                                  endif; ?>>
                                    <?php echo esc_html__('Product Length', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_width" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_width');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Width', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_height" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_height');
                                                                  endif; ?>>
                                    <?php echo esc_html__('Product Height', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_enable_reviews" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_enable_reviews');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Enable Reviews', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_purchase_note" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_purchase_note');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Purchase Note', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_sale_price" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_sale_price');
                                                                     endif; ?>>
                                    <?php echo esc_html__('Product Sale Price', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_regular_price" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_regular_price');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Regular Price', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_shipping_class" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_shipping_class');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Shipping Class', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_position" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_position');
                                                                  endif; ?>>
                                    <?php echo esc_html__('Product Position', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_upsell_ids" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_upsell_ids');
                                                                     endif; ?>>
                                    <?php echo esc_html__('Product Upsell Ids', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_crosssell_ids" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_crosssell_ids');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Crosssell Ids', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_barcode" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_barcode');
                                                                  endif; ?>>
                                    <?php echo esc_html__('Product Barcode', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_download_limit" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_download_limit');
                                                                        endif; ?>>
                                    <?php echo esc_html__('Product Download Limit', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_download_expiry" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_download_expiry');
                                                                           endif; ?>>
                                    <?php echo esc_html__('Product Download Expiry', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_download_file_name" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_download_file_name');
                                                                              endif; ?>>
                                    <?php echo esc_html__('Product Download File Name', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_download_file_url" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_download_file_url');
                                                                           endif; ?>>
                                    <?php echo esc_html__('Product Download File URL', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_wpcom_is_markdown" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_wpcom_is_markdown');
                                                                           endif; ?>>
                                    <?php echo esc_html__('Product WPCOM is Markdown', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <option value="product_brand" <?php if (isset($unique_column_names[$column_name])) : selected($column_name, 'product_brand');
                                                               endif; ?>>
                                    <?php echo esc_html__('Product Brand', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                                 <?php
                                 // Retrieve attribute name
                                 $attribute = CIPFA_PostProcessor::cipfa_extract_attribute($column_name);
                                 ?>
                                 <option value="<?php echo esc_attr('product_' . $attribute . '_attribute'); ?>" <?php if (isset($unique_column_names[$column_name]) && $column_name === 'product_' . $attribute . '_attribute') : selected($column_name, 'product_' . $attribute . '_attribute');
                                                                                                                  endif; ?>>
                                    <?php
                                    echo esc_html__('Product ', 'csv-importer-plus-for-acf') . esc_html(ucwords(CIPFA_PostProcessor::cipfa_clean_attribute_name($attribute)));
                                    ?>
                                 </option>
                                 <option value="<?php echo esc_attr('product_' . $attribute . '_visible'); ?>" <?php if (isset($unique_column_names[$column_name]) && $column_name === 'product_' . $attribute . '_visible') : selected($column_name, 'product_' . $attribute . '_visible');
                                                                                                               endif; ?>>
                                    <?php
                                    echo esc_html__('Product ', 'csv-importer-plus-for-acf') .
                                       esc_html(ucwords(CIPFA_PostProcessor::cipfa_clean_attribute_name($attribute))) .
                                       esc_html__(' Visible', 'csv-importer-plus-for-acf');
                                    ?>
                                 </option>
                                 <option value="<?php echo esc_attr('product_' . $attribute . '_global'); ?>" <?php if (isset($unique_column_names[$column_name]) && $column_name === 'product_' . $attribute . '_global') : selected($column_name, 'product_' . $attribute . '_global');
                                                                                                               endif; ?>>
                                    <?php echo esc_html__('Product ', 'csv-importer-plus-for-acf') .
                                       esc_html(ucwords(CIPFA_PostProcessor::cipfa_clean_attribute_name($attribute))) .
                                       esc_html__(' Global', 'csv-importer-plus-for-acf'); ?>
                                 </option>
                              </optgroup>
                           <?php } ?>

                           <?php //check if ACF exists
                           if (class_exists('ACF')) { ?>
                              <optgroup label="ACF Fields">
                                 <?php
                                 // Get all ACF field groups
                                 $acf_field_groups = acf_get_field_groups();

                                 $field_groups_for_post_type = array();

                                 // Filter field groups based on the selected post type
                                 foreach ($acf_field_groups as $group) {
                                    if (isset($group['location'])) {
                                       foreach ($group['location'] as $locations) {
                                          foreach ($locations as $location) {
                                             if (
                                                ($location['param'] == 'post_type' && $location['operator'] == '==' && $location['value'] == $post_type) ||
                                                ($post_type == 'page' && in_array($location['param'], array('page_template', 'page_type', 'page_parent', 'page'))) ||
                                                ($location['param'] == 'post_template') ||
                                                ($location['param'] == 'post_status') ||
                                                ($location['param'] == 'post_format') ||
                                                ($location['param'] == 'post_category') ||
                                                ($location['param'] == 'post_taxonomy') ||
                                                ($location['param'] == 'post') ||
                                                ($location['param'] == 'current_user') ||
                                                ($location['param'] == 'current_user_role') ||
                                                ($location['param'] == 'user_form') ||
                                                ($location['param'] == 'user_role') ||
                                                ($location['param'] == 'taxonomy') ||
                                                ($location['param'] == 'attachment' && $post_type == 'attachment') ||
                                                ($location['param'] == 'comment') ||
                                                ($location['param'] == 'widget') ||
                                                ($location['param'] == 'nav_menu') ||
                                                ($location['param'] == 'nav_menu_item') ||
                                                ($location['param'] == 'block') ||
                                                ($location['param'] == 'options_page')
                                             ) {
                                                $field_groups_for_post_type[] = $group;
                                                break 2; // No need to check other locations if we found a match
                                             }
                                          }
                                       }
                                    }
                                 }

                                 if (!empty($field_groups_for_post_type)) {
                                    foreach ($field_groups_for_post_type as $group) {
                                       $fields = acf_get_fields($group['key']);

                                       $free_fields = [];
                                       $pro_fields = [];

                                       if ($fields) {
                                          foreach ($fields as $field) {
                                             // Skip specific field types
                                             if (in_array($field['type'], ['message', 'tab', 'accordion', 'group', 'clone'])) {
                                                continue;
                                             }
                                             $field_type = $field['type'];
                                             $is_pro_only = false;

                                             // Check if the field type is restricted to ACF Pro
                                             switch ($field_type) {
                                                case 'file':
                                                case 'link':
                                                case 'post_object':
                                                case 'relationship':
                                                case 'flexible_content':
                                                case 'gallery':
                                                case 'repeater':
                                                case 'checkbox':
                                                case 'radio':
                                                case 'button_group':
                                                case 'wysiwyg':
                                                case 'oembed':
                                                case 'page_link':
                                                case 'taxonomy':
                                                case 'user':
                                                case 'google_map':
                                                case 'date_time_picker':
                                                case 'time_picker':
                                                case 'color_picker':
                                                case 'icon_picker':
                                                   $is_pro_only = true;
                                                   break;
                                                default:
                                                   $is_pro_only = false;
                                                   break;
                                             }

                                             // Add the field to the appropriate array
                                             if ($is_pro_only) {
                                                $pro_fields[] = $field;
                                             } else {
                                                $free_fields[] = $field;
                                             }
                                          }

                                          // Display free fields first
                                          foreach ($free_fields as $field) {
                                             echo '<option value="' . esc_attr($field['key']) . '" ' . (isset($unique_column_names[$column_name]) ? selected($column_name, $field['name'], false) : '') . '>' . esc_html(ucwords($field['label'])) . '</option>';
                                          }

                                          // Display Pro fields below with a note
                                          foreach ($pro_fields as $field) {
                                             echo '<option value="' . esc_attr($field['key']) . '" disabled style="color: #999;">' . esc_html(ucwords($field['label'])) . ' - Pro Access</option>';
                                          }
                                       } else {
                                          echo '<option disabled>No ACF fields found in group ' . esc_html($group['title']) . '</option>';
                                       }
                                    }
                                 } else {
                                    echo '<option disabled>' . esc_html__('No ACF field groups found for this post type', 'csv-importer-plus-for-acf') . '</option>';
                                 }
                                 ?>
                              </optgroup>
                           <?php } ?>
                        </select>
                     </td>
                  </tr>
               <?php endforeach; ?>
            </tbody>
         </table>
      </div>
      <div class="flex justify-end">
         <button type="submit" class="import-button inline-flex items-center px-4 py-2 border border-transparent text-sm font-medium rounded-md text-white bg-gradient-to-br from-blue-600 to-purple-600 hover:bg-indigo-600 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500" name="import_data"><?php echo esc_html__('Import Data', 'csv-importer-plus-for-acf') ?></button>
      </div>
   </form>
</div>
<p class="text-center m-4">
   <?php esc_html_e('Please', 'csv-importer-plus-for-acf'); ?>
   <a href=" <?php echo esc_url('https://wordpress.org/support/plugin/csv-importer-plus-for-acf/reviews/#new-post'); ?>" target="_blank" title="<?php esc_attr_e('Rate the plugin', 'csv-importer-plus-for-acf'); ?>">
      <?php esc_html_e('Rate the plugin', 'csv-importer-plus-for-acf'); ?>
      <span class="rate-star">★★★★★</span>
   </a><?php esc_html_e('to help us spread the word. Thank you from the CSV Importer Plus for ACF team!', 'csv-importer-plus-for-acf'); ?>
</p>